//******************************************************************
// Exam program
// This program compares students who come to take an exam against
// a class roster.  A list of students who took the exam and a list
// of students who missed the exam are printed.
// Assumption: Max. number of student names in roster file is
// MAX_LENGTH, which is defined in slist.h
//******************************************************************
#include <iostream>
#include <fstream>      // For file I/O
#include <string>       // For string class
#include "slist.h"      // For SortedList class

const string END_DATA = "EndData";    // Sentinel value for
                                      //   student name

void CheckInStudents( SortedList&, SortedList& );
void GetClassRoster( SortedList&, ifstream& );
void OpenForInput( ifstream& );
void Print( SortedList, SortedList );
void ProcessName( string, SortedList&, SortedList& );

int main()
{
    SortedList notCheckedIn;  // List of students not yet checked in
    SortedList thosePresent;  // List of students present at exam
    ifstream roster;          // Input file to be analyzed

    OpenForInput(roster);
    if ( !roster )
        return 1;

    GetClassRoster(notCheckedIn, roster);
    CheckInStudents(notCheckedIn, thosePresent);
    Print(notCheckedIn, thosePresent);
    return 0;
}

//******************************************************************

void OpenForInput( /* inout */ ifstream& someFile )    // File to be
                                                       // opened
// Prompts the user for the name of an input file
// and attempts to open the file

// Postcondition:
//     The user has been prompted for a file name
//  && IF the file could not be opened
//         An error message has been printed
// Note:
//     Upon return from this function, the caller must test
//     the stream state to see if the file was successfully opened

{
    string fileName;    // User-specified file name

    cout << "Input file name: ";
    cin >> fileName;

    someFile.open(fileName.c_str());
    if ( !someFile )
        cout << "** Can't open " << fileName << " **" << endl;
}

//******************************************************************

void GetClassRoster(
        /* inout */ SortedList& notCheckedIn,    // List of students
        /* inout */ ifstream&   roster       )   // Roster data file

// Reads the class roster from the data file

// Precondition:
//     notCheckedIn is the empty list
// Precondition:
//     The roster file has been successfully opened for input
//  && The no. of student names in the file <= MAX_LENGTH (defined
//       in slist.h)
// Postcondition:
//     notCheckedIn contains the student names read from the file

{
    string stuName;      // An input student name

    roster >> stuName;
    while (roster)
    {
        notCheckedIn.Insert(stuName);
        roster >> stuName;
    }
}

//******************************************************************

void CheckInStudents(
      /* inout */ SortedList& notCheckedIn,     // List of students
      /* inout */ SortedList& thosePresent )    // List of students

// Inputs student names from standard input,
// marking students present

// Precondition:
//     notCheckedIn contains names of all students in class
//  && thosePresent is empty
// Postcondition:
//     The user has been repeatedly prompted to enter student names
//  && thosePresent contains all the valid input names
//  && notCheckedIn contains all student names except the valid
//       input names
//  && For all input names not found in the notCheckedIn list,
//     an error message has been printed

{
    string stuName;    // Name of student who is checking in

    cout << "Enter last name: ";
    cin >> stuName;
    while (stuName != END_DATA)
    {
        ProcessName(stuName, notCheckedIn, thosePresent);
        cout << "Enter last name: ";
        cin >> stuName;
    }
}

//******************************************************************

void ProcessName(
      /* in */    string      stuName,         // Input student name
      /* inout */ SortedList& notCheckedIn,    // List of students
      /* inout */ SortedList& thosePresent )   // List of students

// Searches for stuName in the notCheckedIn list.  If stuName
// is found, it is inserted into thosePresent and deleted from
// notCheckedIn.  Otherwise, an error message is printed

// Precondition:
//     stuName is assigned
// Postcondition:
//     IF stuName is in notCheckedIn@entry
//         stuName is in thosePresent
//      && stuName is not in notCheckedIn
//     ELSE
//         An error message has been printed

{
    if (notCheckedIn.IsPresent(stuName))
    {
        thosePresent.Insert(stuName);
        notCheckedIn.Delete(stuName);
    }
    else
        cout << "Name not on roster." << endl;
}

//******************************************************************

void Print( /* in */ SortedList notCheckedIn,    // List of students
            /* in */ SortedList thosePresent )   // List of students

// Prints the names of those taking the exam, then the names
// of those who are absent

// Postcondition:
//     Contents of thosePresent and notCheckedIn have been printed

{
    cout << endl << "The following students are taking the exam."
         << endl;
    thosePresent.Print();

    cout << endl << "The following students have missed the exam."
         << endl;
    notCheckedIn.Print();
}
