//******************************************************************
// SPECIFICATION FILE (list.h)
// This file gives the specification of a list abstract data type.
// The list components are not assumed to be in order by value
//******************************************************************

const int MAX_LENGTH = 50;      // Maximum possible number of
                                //   components needed
typedef int ItemType;           // Type of each component
                                //   (a simple type or string class)

class List
{
public:
    bool IsEmpty() const;
        // Postcondition:
        //     Function value == true, if list is empty
        //                    == false, otherwise

    bool IsFull() const;
        // Postcondition:
        //     Function value == true, if list is full
        //                    == false, otherwise

    int Length() const;
        // Postcondition:
        //     Function value == length of list

    void Insert( /* in */ ItemType item );
        // Precondition:
        //     NOT IsFull()
        //  && item is assigned
        // Postcondition:
        //     item is in list
        //  && Length() == Length()@entry + 1

    void Delete( /* in */ ItemType item );
        // Precondition:
        //     NOT IsEmpty()
        //  && item is assigned
        // Postcondition:
        //     IF item is in list at entry
        //           First occurrence of item is no longer in list
        //        && Length() == Length()@entry - 1
        //     ELSE
        //           List is unchanged

    bool IsPresent( /* in */ ItemType item ) const;
        // Precondition:
        //     item is assigned
        // Postcondition:
        //     Function value == true, if item is in list
        //                    == false, otherwise

    void SelSort();
        // Postcondition:
        //     List components are in ascending order of value

    void Print() const;
        // Postcondition:
        //     All components (if any) in list have been output

    List();
        // Constructor
        // Postcondition:
        //     Empty list is created
private:
    int      length;
    ItemType data[MAX_LENGTH];
};
