//******************************************************************
// SortWithPointers program
// This program reads personnel records from a data file,
// sorts the records alphabetically by last name,
// and writes them to the standard output device
// Assumption: The input file contains at most 1000 records
//******************************************************************
#include <iostream>
#include <fstream>   // For file I/O
#include <string>    // For string class (for OpenForInput function)
#include "reclist.h" // For RecordList class

using namespace std;

void OpenForInput( ifstream& );

int main()
{
    RecordList list;           // List of personnel records
    ifstream   masterFile;     // Input file of personnel records

    OpenForInput(masterFile);
    if ( !masterFile )
        return 1;

    list.ReadAll(masterFile);
    list.SelSort();
    list.PrintAll();
    return 0;
}

//******************************************************************

void OpenForInput( /* inout */ ifstream& someFile )    // File to be
                                                       // opened
// Prompts the user for the name of an input file
// and attempts to open the file

// Postcondition:
//     The user has been prompted for a file name
//  && IF the file could not be opened
//         An error message has been printed
// Note:
//     Upon return from this function, the caller must test
//     the stream state to see if the file was successfully opened

{
    string fileName;    // User-specified file name

    cout << "Input file name: ";
    cin >> fileName;

    someFile.open(fileName.c_str());
    if ( !someFile )
        cout << "** Can't open " << fileName << " **" << endl;
}
